from __future__ import absolute_import
from __future__ import print_function
import json
import wrapt
from .io import error
import sys
import traceback


@wrapt.decorator
def custom_exception(wrapped, instance, args, kwargs):
    try:
        return wrapped(*args, **kwargs)
    except Exception as e:
        if hasattr(e, 'body'):
            try:
                error(json.loads(e.body)['err_msg'])
            except json.decoder.JSONDecodeError:
                error(str(e))
        else:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
            error(''.join(lines))
            error(str(e))
            ctx = args[0]
            ctx.exit(1)


@wrapt.decorator
def list_output(wrapped, instance, args, kwargs):
    output = wrapped(*args, **kwargs)
    print((json.dumps(output, indent=4)))


@wrapt.decorator
def dict_output(wrapped, instance, args, kwargs):
    output = wrapped(*args, **kwargs)
    print((json.dumps(output, indent=4)))


@wrapt.decorator
def str_output(wrapped, instance, args, kwargs):
    print(wrapped(*args, **kwargs))


@wrapt.decorator
def None_output(wrapped, instance, args, kwargs):
    print(wrapped(*args, **kwargs))


def _arg_split(ctx, param, value):
    # split columns by ',' and remove whitespace
    columns = [c.strip() for c in value.split(',')]
    return columns



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from baricadr.exceptions import BaricadrConnectionError
from baricadr.file import FileClient
from baricadr.task import TaskClient

from future import standard_library

import requests

standard_library.install_aliases()


class BaricadrInstance(object):

    def __init__(self, host="localhost", port="9100", login=None, password=None, **kwargs):
        self.host = host
        self.port = str(port)

        self.login = login
        self.password = password

        self.endpoints = self._get_endpoints()

        # Initialize Clients
        args = (self.host, self.port, self.login, self.password, self.endpoints)
        self.file = FileClient(*args)
        self.task = TaskClient(*args)

    def __str__(self):
        return '<BarricadrInstance at {}:{}>'.format(self.host, self.port)

    def _get_endpoints(self):

        auth = None
        if self.login and self.password:
            auth = (self.login, self.password)

        try:
            r = requests.get("http://{}:{}/endpoints".format(self.host, self.port), auth=auth)
            if not r.status_code == 200:
                raise requests.exceptions.RequestException
            return r.json()
        except requests.exceptions.RequestException:
            raise BaricadrConnectionError("Cannot connect to {}:{}. Please check the connection.".format(self.host, self.port))



import click
from barique.commands.file.freeze import cli as freeze
from barique.commands.file.list import cli as list
from barique.commands.file.pull import cli as pull


@click.group()
def cli():
    """
    Manipulate files managed by Baricadr
    """
    pass


cli.add_command(freeze)
cli.add_command(list)
cli.add_command(pull)



import click
from barique.cli import pass_context, json_loads
from barique.decorators import custom_exception, list_output


@click.command('list')
@pass_context
@custom_exception
@list_output
def cli(ctx):
    """List recent tasks

Output:

    List of tasks
    """
    return ctx.gi.task.list()



import click
from barique.cli import pass_context, json_loads
from barique.decorators import custom_exception, str_output


@click.command('freeze')
@click.argument("path", type=str)
@click.option(
    "--force",
    help="Force freezing, even if the freezing delay was not reached",
    is_flag=True
)
@click.option(
    "--dry_run",
    help="Do not make any deletion, just list changes that would be made",
    is_flag=True
)
@click.option(
    "--email",
    help="User email adress for notification",
    type=str
)
@pass_context
@custom_exception
@str_output
def cli(ctx, path, force=False, dry_run=False, email=""):
    """Launch a freeze task

Output:

    Id associated to the freeze task
    """
    return ctx.gi.file.freeze(path, force=force, dry_run=dry_run, email=email)



import click
from barique.cli import pass_context, json_loads
from barique.decorators import custom_exception, str_output


@click.command('pull')
@click.argument("path", type=str)
@click.option(
    "--email",
    help="User email adress for notification",
    type=str
)
@click.option(
    "--dry_run",
    help="Do not make any pull, just list changes that would be made",
    is_flag=True
)
@pass_context
@custom_exception
@str_output
def cli(ctx, path, email="", dry_run=False):
    """Launch a pull task

Output:

    Id associated to the pull task
    """
    return ctx.gi.file.pull(path, email=email, dry_run=dry_run)



import click
from barique.commands.task.list import cli as list
from barique.commands.task.log import cli as log
from barique.commands.task.remove import cli as remove
from barique.commands.task.show import cli as show
from barique.commands.task.zombies import cli as zombies


@click.group()
def cli():
    """
    Track progress of Baricadr tasks
    """
    pass


cli.add_command(list)
cli.add_command(log)
cli.add_command(remove)
cli.add_command(show)
cli.add_command(zombies)



from setuptools import find_packages, setup

with open('requirements.txt') as f:
    requires = f.read().splitlines()

setup(
    name="barique",
    version='0.1.2',
    description="Baricadr CLI",
    author="Mateo Boudet",
    author_email="mateo.boudet@inrae.fr",
    url="https://github.com/baricadr/barique",
    install_requires=requires,
    packages=find_packages(),
    license='MIT',
    platforms="Posix; MacOS X; Windows",
    entry_points='''
        [console_scripts]
        barique=barique.cli:barique
    ''',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
        "Programming Language :: Python :: 3.7",
    ]
)



import click
from barique.cli import pass_context, json_loads
from barique.decorators import custom_exception, dict_output


@click.command('remove')
@click.argument("task_id", type=str)
@pass_context
@custom_exception
@dict_output
def cli(ctx, task_id):
    """Remove task with the selected id

Output:

    Dict
    """
    return ctx.gi.task.remove(task_id)



import click
from barique.cli import pass_context, json_loads
from barique.decorators import custom_exception, list_output


@click.command('list')
@click.argument("path", type=str)
@click.option(
    "--full",
    help="List full information for each file (size (in bytes), mtime, etc..)",
    is_flag=True
)
@click.option(
    "--missing",
    help="Only list files missing from the local path",
    is_flag=True
)
@click.option(
    "--max_depth",
    help="Restrict to a max depth. Set to 0 for all files.",
    default="1",
    show_default=True,
    type=int
)
@click.option(
    "--from_root",
    help="Return full paths from root of the repo (instead of relative to given path)",
    is_flag=True
)
@pass_context
@custom_exception
@list_output
def cli(ctx, path, full=False, missing=False, max_depth=1, from_root=False):
    """List files available from a remote repository for a local path

Output:

    List of file relative paths
    """
    return ctx.gi.file.list(path, full=full, missing=missing, max_depth=max_depth, from_root=from_root)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from baricadr.client import Client

from future import standard_library

standard_library.install_aliases()


class TaskClient(Client):
    """
    Track progress of Baricadr tasks
    """

    def list(self):
        """
        List recent tasks

        :rtype: list
        :return: List of tasks
        """

        return self._api_call("get", "task_list")

    def show(self, task_id):
        """
        Show task with the specified id

        :type task_id: str
        :param task_id: Task id

        :rtype: dict
        :return: Dict
        """

        args = {"task_id": str(task_id)}

        return self._api_call("get", "task_show", args)

    def log(self, task_id):
        """
        Show log from the task with the specified id

        :type task_id: str
        :param task_id: Task id

        :rtype: dict
        :return: Dict
        """

        args = {"task_id": str(task_id)}

        return self._api_call("get", "task_log", args)

    def remove(self, task_id):
        """
        Remove task with the selected id

        :type task_id: str
        :param task_id: Task id

        :rtype: dict
        :return: Dict
        """

        args = {"task_id": str(task_id)}

        return self._api_call("get", "task_remove", args)

    def zombies(self):
        """
        Kill zombie tasks

        :rtype: dict
        :return: Task id
        """

        return self._api_call("get", "zombie")



import click
from barique.cli import pass_context, json_loads
from barique.decorators import custom_exception, dict_output


@click.command('zombies')
@pass_context
@custom_exception
@dict_output
def cli(ctx):
    """Kill zombie tasks

Output:

    Task id
    """
    return ctx.gi.task.zombies()



# coding: utf-8
import os

import click

from baricadr import BaricadrInstance
from barique.cli import pass_context
from barique import config
from barique.io import warn, info

CONFIG_TEMPLATE = """## Baricadr's barique: Global Configuration File.
# Each stanza should contain a single baricadr server to control.
#
# You can set the key __default to the name of a default instance
__default: local
local:
    host: "%(host)s"
    port: "%(port)s"
"""

CONFIG_TEMPLATE_AUTH = """## Baricadr's barique: Global Configuration File.
# Each stanza should contain a single baricadr server to control.
#
# You can set the key __default to the name of a default instance
__default: local
local:
    host: "%(host)s"
    port: "%(port)s"
    login: "%(login)s"
    password: "%(password)s"
"""

SUCCESS_MESSAGE = (
    "Ready to go! Type `barique` to get a list of commands you can execute."
)


@click.command("config_init")
@pass_context
def cli(ctx, url=None, api_key=None, admin=False, **kwds):
    """Help initialize global configuration (in home directory)
    """

    click.echo("""Welcome to Barique""")
    if os.path.exists(config.global_config_path()):
        info("Your barique configuration already exists. Please edit it instead: %s" % config.global_config_path())
        return 0

    while True:
        # Check environment
        host = click.prompt("Baricadr server host")
        port = click.prompt("Baricadr server port")
        login = None
        password = None
        if click.confirm("""Is your Baricadr instance running behind an authentication proxy?"""):
            login = click.prompt("Baricadr username")
            password = click.prompt("Baricadr password", hide_input=True)

        info("Testing connection...")
        try:
            BaricadrInstance(host=host, port=port, login=login, password=password)
            # We do a connection test during startup.
            info("Ok! Everything looks good.")
            break
        except Exception as e:
            warn("Error, we could not access the configuration data for your instance: %s", e)
            should_break = click.prompt("Continue despite inability to contact this instance? [y/n]")
            if should_break in ('Y', 'y'):
                break

    config_path = config.global_config_path()
    if os.path.exists(config_path):
        warn("File %s already exists, refusing to overwrite." % config_path)
        return -1

    with open(config_path, "w") as f:
        if login and password:
            f.write(CONFIG_TEMPLATE_AUTH % {
                'host': host,
                'port': port,
                'login': login,
                'password': password,
            })
        else:
            f.write(CONFIG_TEMPLATE % {
                'host': host,
                'port': port,
            })
        info(SUCCESS_MESSAGE)

    # We don't want other users to look into this file
    os.chmod(config_path, 0o600)
